import java.util.Random;

/**
 * Vergleicht zwei Methoden, um ein int[] Array aufzusummieren. Mittels
 * for-Schleife oder mittels try-catch und while-Schleife.
 */
public class FastSum {

	private int[] values;
	
	/**
	 * @param n - Grsse des Arrays
	 */
	public FastSum(int n) {
		values = new int[n];
		
		// Flle das Array mit Zufallszahlen
		Random rand = new Random();
		for (int i=0;i<n;i++) {
			values[i] = rand.nextInt();
		}
	}

	/**
	 * Summiere die Elemente des Arrays mittels for-Schleife und gebe die dafr
	 * bentigte Zeit zurck.
	 */
	public long sum() {
		long t0 = System.nanoTime();
		
		int sum = 0;
		for (int i=0;i<values.length;i++) {
			sum += values[i];
		}
		
		long dt = System.nanoTime()-t0;
		return dt;
	}

	/**
	 * Summiere die Elemente des Arrays innerhalb einer while-Schleife bis eine
	 * ArrayIndexoutOfBoundsException auftritt und fange diese dann ab. Gib die
	 * dafr bentigte Zeit zurck.
	 */
	public long fastsum() {
		long t0 = System.nanoTime();
		
		int sum = 0;
		try {
			int i = 0;
			while(true) {
				sum += values[i++];
			}
		} catch (ArrayIndexOutOfBoundsException e) { }
		
		long dt = System.nanoTime()-t0;
		return dt;
	}
	
	public static void main(String[] args) {
		int n = 100000; // Grsse des Arrays
		FastSum instance = new FastSum(n);

		// Messe die schnellste Zeit fr sum aus 100 Versuchen
		long sumMin = Long.MAX_VALUE;
		for (int i=0;i<100;i++) {
			sumMin = Math.min(sumMin, instance.sum());
		}
		
		// Messe die schnellste Zeit fr fastsum aus 100 Versuchen
		long fsMin = Long.MAX_VALUE;
		for (int i=0;i<100;i++) {
			fsMin = Math.min(fsMin, instance.fastsum());
		}
		
		System.out.println("sum:     "+sumMin/1000+" mms");
		System.out.println("fastsum: "+fsMin/1000+" mms");
	}

}
